"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractController = void 0;
const error_keys_enum_1 = require("../../enum/error-keys.enum");
const field_schema_builder_1 = require("../../helper/validator/field-schema.builder");
const schema_validator_helper_1 = require("../../helper/validator/schema-validator.helper");
const abstract_entity_loader_1 = require("../../loader/abstract-entity.loader");
class AbstractController extends abstract_entity_loader_1.AbstractEntityLoader {
    constructor(entity, fieldSchemaData, entitySchemaData, errorService, translationService) {
        super(entity, fieldSchemaData, entitySchemaData);
        this.entity = entity;
        this.fieldSchemaData = fieldSchemaData;
        this.entitySchemaData = entitySchemaData;
        this.errorService = errorService;
        this.translationService = translationService;
        this.requestSchema = new field_schema_builder_1.FieldSchemaBuilder(errorService, translationService, entity, entitySchemaData).buildRequestSchemas(this.fieldSchemaDb, this.fieldSchemaData);
        this.schemaValidator = new schema_validator_helper_1.SchemaValidator(errorService, translationService, this.entitySchema, this.entitySchemaData);
    }
    isMethodAllowed(method) {
        if (!this.entitySchema.forbiddenMethods)
            return;
        const notAllowed = this.entitySchema.forbiddenMethods.filter((m) => m === method);
        if (notAllowed.length)
            this.errorService.throwError(error_keys_enum_1.ErrorKeys.METHOD_NOT_ALLOWED);
    }
}
exports.AbstractController = AbstractController;
