"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComparatorHelper = void 0;
const get_dynamic_value_service_1 = require("../../service/dynamic/get-dynamic-value.service");
const nestjs_microservices_schemas_1 = require("@devseeder/nestjs-microservices-schemas");
const error_keys_enum_1 = require("../../enum/error-keys.enum");
class ComparatorHelper {
    constructor(errorService, translationService, entityLabels = []) {
        this.errorService = errorService;
        this.translationService = translationService;
        this.entityLabels = entityLabels;
    }
    static compare(valueA, valueB, operator, type) {
        switch (type) {
            case 'text':
            case 'varchar':
            case 'string':
                valueA = String(valueA);
                valueB = String(valueB);
            case 'number':
            case 'double':
            case 'integer':
                valueA = Number(valueA);
                valueB = Number(valueB);
                break;
            default:
                return ComparatorHelper.compareValues(valueA, valueB, operator);
        }
    }
    static compareValues(valueA, valueB, operator) {
        switch (operator) {
            case nestjs_microservices_schemas_1.CompareOperators.GREATER_THAN:
                return valueA > valueB;
            case nestjs_microservices_schemas_1.CompareOperators.GREATER_THAN_EQUAL:
                return valueA >= valueB;
            case nestjs_microservices_schemas_1.CompareOperators.LOWER_THAN:
                return valueA < valueB;
            case nestjs_microservices_schemas_1.CompareOperators.LOWER_THAN_EQUAL:
                return valueA <= valueB;
            case nestjs_microservices_schemas_1.CompareOperators.NOT_EQUAL:
                return valueA != valueB;
            case nestjs_microservices_schemas_1.CompareOperators.EQUAL:
                return valueA == valueB;
            case nestjs_microservices_schemas_1.CompareOperators.IN:
                return valueB.includes(valueA);
            case nestjs_microservices_schemas_1.CompareOperators.NOT_IN:
                return !valueB.includes(valueA);
            case nestjs_microservices_schemas_1.CompareOperators.LIKE:
                return valueA.indexOf(valueB) > -1;
            case nestjs_microservices_schemas_1.CompareOperators.BETWEEN:
                return valueA >= valueB[0] && valueA <= valueB[1];
            case nestjs_microservices_schemas_1.CompareOperators.BEGINS_WITH:
                return valueA.startsWith(valueB);
            case nestjs_microservices_schemas_1.CompareOperators.ENDS_WITH:
                return valueA.endsWith(valueB);
            case nestjs_microservices_schemas_1.CompareOperators.CONTAINS:
                return valueA.includes(valueB);
            case nestjs_microservices_schemas_1.CompareOperators.NOT_CONTAINS:
                return !valueA.includes(valueB);
        }
    }
    getComparatorError(key, value, operator) {
        return __awaiter(this, void 0, void 0, function* () {
            const valueB = new get_dynamic_value_service_1.DynamicValueService(this.translationService).getValueMessage(value, value);
            const compKey = yield this.translationService.getFieldTranslation(this.entityLabels, key);
            switch (operator) {
                case nestjs_microservices_schemas_1.CompareOperators.GREATER_THAN:
                case nestjs_microservices_schemas_1.CompareOperators.GREATER_THAN_EQUAL:
                case nestjs_microservices_schemas_1.CompareOperators.LOWER_THAN:
                case nestjs_microservices_schemas_1.CompareOperators.LOWER_THAN_EQUAL:
                case nestjs_microservices_schemas_1.CompareOperators.NOT_EQUAL:
                case nestjs_microservices_schemas_1.CompareOperators.EQUAL:
                case nestjs_microservices_schemas_1.CompareOperators.LIKE:
                case nestjs_microservices_schemas_1.CompareOperators.BEGINS_WITH:
                case nestjs_microservices_schemas_1.CompareOperators.ENDS_WITH:
                case nestjs_microservices_schemas_1.CompareOperators.CONTAINS:
                case nestjs_microservices_schemas_1.CompareOperators.NOT_CONTAINS:
                    this.errorService.throwError(this.getEnumKeyByEnumValue(nestjs_microservices_schemas_1.CompareOperators, operator), {
                        key: compKey.fieldLabel,
                        value: valueB,
                    });
                    break;
                case nestjs_microservices_schemas_1.CompareOperators.IN:
                case nestjs_microservices_schemas_1.CompareOperators.NOT_IN:
                case nestjs_microservices_schemas_1.CompareOperators.BETWEEN:
                    this.errorService.throwError(this.getEnumKeyByEnumValue(nestjs_microservices_schemas_1.CompareOperators, operator), {
                        key: compKey.fieldLabel,
                        value: valueB.join(','),
                    });
                default:
                    this.errorService.throwError(error_keys_enum_1.ErrorKeys.INVALID_DATA, {
                        key: 'Comparator',
                        value: compKey.fieldLabel,
                    });
            }
        });
    }
    getEnumKeyByEnumValue(myEnum, enumValue) {
        const keys = Object.keys(myEnum).filter((x) => myEnum[x] == enumValue);
        return keys.length > 0 ? keys[0] : null;
    }
}
exports.ComparatorHelper = ComparatorHelper;
