"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FieldSchemaBuilder = void 0;
const Joi = __importStar(require("joi"));
const schema_validator_helper_1 = require("./schema-validator.helper");
const nestjs_microservices_schemas_1 = require("@devseeder/nestjs-microservices-schemas");
const nestjs_microservices_schemas_2 = require("@devseeder/nestjs-microservices-schemas");
const error_keys_enum_1 = require("../../enum/error-keys.enum");
const field_schema_interface_1 = require("../../interface/field-schema.interface");
const abstract_input_schema_1 = require("../../field-schemas/abstract-input.schema");
class FieldSchemaBuilder {
    constructor(errorService, translationService, entity, entitySchemaData) {
        this.errorService = errorService;
        this.entitySchemaData = entitySchemaData;
        this.entitySchema = entitySchemaData.filter((ent) => ent.entity == entity)[0];
        this.schemaValidator = new schema_validator_helper_1.SchemaValidator(errorService, translationService, this.entitySchema, entitySchemaData);
    }
    buildRequestSchemas(entityFields, fieldSchemaData) {
        const parentSchemas = {};
        const childrenSchemas = {};
        entityFields
            .filter((field) => field.type === 'externalId')
            .forEach((field) => {
            const relation = field.externalRelation.service;
            const entitySchema = this.entitySchemaData.filter((sc) => sc.entity === relation);
            parentSchemas[relation] = this.buildSchemas(fieldSchemaData.filter((schema) => (entitySchema[0].extendedEntities &&
                entitySchema[0].extendedEntities.includes(schema.entity)) ||
                schema.entity === nestjs_microservices_schemas_1.GLOBAL_ENTITY ||
                schema.entity === relation));
        });
        this.entitySchema.subRelations.forEach((sub) => {
            const relation = sub.service;
            const entitySchema = this.entitySchemaData.filter((sc) => sc.entity === relation);
            childrenSchemas[relation] = this.buildSchemas(fieldSchemaData.filter((schema) => (entitySchema[0].extendedEntities &&
                entitySchema[0].extendedEntities.includes(schema.entity)) ||
                schema.entity === nestjs_microservices_schemas_1.GLOBAL_ENTITY ||
                schema.entity === relation));
        });
        return {
            entity: this.buildSchemas(entityFields),
            parents: parentSchemas,
            children: childrenSchemas,
        };
    }
    buildSchemas(fieldSchema) {
        return {
            search: this.buildSearchSchema(fieldSchema, abstract_input_schema_1.commonSearchSchema),
            update: this.buildUpdateSchema(fieldSchema),
            create: this.buildCreateSchema(fieldSchema),
            cloneOne: this.buildCloneSchema(fieldSchema, abstract_input_schema_1.singleCloneSchema),
            cloneMany: this.buildCloneSchema(fieldSchema, abstract_input_schema_1.manyCloneSchema),
            count: this.buildSearchSchema(fieldSchema),
            groupBy: this.buildSearchSchema(fieldSchema, abstract_input_schema_1.commonGroupBySchema),
            activation: Joi.object(Object.assign({}, abstract_input_schema_1.commonActivationSchema)),
        };
    }
    cleanSchema(fieldSchema) {
        return fieldSchema.filter((sch) => sch.key.indexOf('.') === -1);
    }
    buildSearchSchema(fieldSchema, commons = {}) {
        const objectSchema = {};
        fieldSchema
            .filter((field) => !field.key.startsWith('$') && field.allowed.search)
            .forEach((schema) => {
            if (this.buildSearchEngine(schema, objectSchema))
                return;
            const joiSchema = this.getType(Joi, schema, true);
            if (schema.type === 'enum')
                objectSchema[schema.key] = joiSchema
                    .optional()
                    .custom(this.schemaValidator.validateEnum(schema.enumValues));
            else
                objectSchema[schema.key] = joiSchema.optional();
        });
        return Joi.object(Object.assign(Object.assign({}, commons), objectSchema));
    }
    buildUpdateSchema(fieldSchema) {
        return Joi.object(this.buildObjectUpdate(fieldSchema));
    }
    buildCloneSchema(fieldSchema, cloneSchema) {
        return Joi.object(Object.assign(Object.assign({}, cloneSchema), { replaceBody: this.buildObjectUpdate(fieldSchema) }));
    }
    buildObjectUpdate(fieldSchema) {
        const objectSchema = {};
        this.cleanSchema(fieldSchema)
            .filter((field) => field.allowed.update)
            .forEach((schema) => {
            const joiSchema = this.getType(Joi, schema, false, schema === null || schema === void 0 ? void 0 : schema.array, fieldSchema);
            if (schema.type === 'enum')
                objectSchema[schema.key] = joiSchema
                    .optional()
                    .custom(this.schemaValidator.validateEnum(schema.enumValues));
            else
                objectSchema[schema.key] = joiSchema.optional().allow(null);
        });
        return objectSchema;
    }
    buildCreateSchema(fieldSchema) {
        const objectSchema = {};
        this.cleanSchema(fieldSchema).forEach((schema) => {
            let joiSchema = this.getType(Joi, schema, false, schema === null || schema === void 0 ? void 0 : schema.array, fieldSchema);
            joiSchema = schema.required
                ? joiSchema.required()
                : joiSchema.optional().allow(null);
            if (schema.type === 'enum')
                objectSchema[schema.key] = joiSchema
                    .optional()
                    .allow(null)
                    .custom(this.schemaValidator.validateEnum(schema.enumValues));
            else
                objectSchema[schema.key] = joiSchema;
        });
        return Joi.object(objectSchema);
    }
    getType(Joi, schema, search = false, array = false, schemasDb = []) {
        switch (schema.type) {
            case 'text':
            case 'string':
            case 'password':
            case 'image':
            case 'link':
                return Joi.string();
            case 'email':
                return Joi.string().email();
            case 'externalId':
                return array ? Joi.array() : Joi.string();
            case 'enum':
                return schema.itensType === 'string' ? Joi.string() : Joi.number();
            case 'boolean':
                return Joi.boolean();
            case 'number':
            case 'currency':
            case 'decimal':
            case 'double':
            case 'integer':
                let joiSchema = Joi.number();
                if (schema.min !== undefined)
                    joiSchema = joiSchema.min(schema.min);
                if (schema.max !== undefined)
                    joiSchema = joiSchema.max(schema.max);
                return joiSchema;
            case 'date':
            case 'datetime':
                return Joi.alternatives().try(Joi.date().iso(), Joi.string().regex(/^@/));
            case 'array':
                return search
                    ? Joi.string()
                    : this.buildArrayField(Joi, schema, search, schemasDb);
            case 'translation':
            case 'object':
            case 'json':
                const schemaObj = this.buildObjectFieldSchema(Joi, schema, search, schemasDb);
                return schemaObj ? Joi.object(schemaObj) : Joi.object();
            case 'file':
                return schema.array ? Joi.array() : Joi.object();
            default:
                this.errorService.throwError(error_keys_enum_1.ErrorKeys.NOT_IMPLEMENTED, {
                    key: 'Schema type',
                    value: schema.type,
                });
        }
    }
    buildArrayField(Joi, schema, search = false, schemasDb = []) {
        if (schema.itensType !== 'object')
            return Joi.array();
        const schemaObj = this.buildObjectFieldSchema(Joi, schema, search, schemasDb);
        if (!schemaObj)
            return Joi.array();
        return Joi.array().items(Joi.object(schemaObj));
    }
    buildObjectFieldSchema(Joi, schema, search = false, schemasDb = []) {
        if (!schemasDb.length)
            return null;
        const propObjFields = schemasDb.filter((sch) => sch.key.startsWith(`${schema.key}.`));
        if (!propObjFields.length)
            return null;
        const objSchema = {};
        propObjFields.forEach((prop) => {
            let propSchema = this.getType(Joi, prop, search, prop.array, schemasDb);
            if (prop.required)
                propSchema = propSchema.required();
            objSchema[prop.key.replace(`${schema.key}.`, '')] = propSchema;
        });
        return objSchema;
    }
    buildSearchEngine(schema, objectSchema) {
        if (!(schema === null || schema === void 0 ? void 0 : schema.searchEgines))
            return false;
        let ignoreOriginalKey = false;
        if (schema === null || schema === void 0 ? void 0 : schema.searchEgines.includes(nestjs_microservices_schemas_2.SearchEgineOperators.BETWEEN)) {
            const start = this.getType(Joi, schema, true);
            const end = this.getType(Joi, schema, true);
            objectSchema[`${schema.key}_start`] = start.optional();
            objectSchema[`${schema.key}_end`] = end.optional();
            ignoreOriginalKey = true;
        }
        Object.values(nestjs_microservices_schemas_2.SearchEgineOperators).forEach((op) => {
            ignoreOriginalKey = false;
            const joiSchema = this.getType(Joi, schema, true);
            objectSchema[`${schema.key}_${op}`] = joiSchema.optional();
        });
        if (schema.type == 'boolean') {
            const joiSchema = this.getType(Joi, schema, true);
            objectSchema[`${schema.key}_ne`] = joiSchema.optional();
        }
        return ignoreOriginalKey;
    }
    getFormFilterCondition(page, field) {
        switch (page) {
            case field_schema_interface_1.FieldSchemaPage.SEARCH:
                return field.allowed.search;
            case field_schema_interface_1.FieldSchemaPage.UPDATE:
                return field.allowed.update;
            case field_schema_interface_1.FieldSchemaPage.CREATE:
                return true;
            default:
                this.errorService.throwError(error_keys_enum_1.ErrorKeys.INVALID_DATA, {
                    key: 'page',
                    value: page,
                });
        }
    }
    static checkUndefinedValue(value, schema, itemResponse, operator) {
        if (value === undefined && nestjs_microservices_schemas_1.SKIP_ENUMS_ALIAS.includes(operator))
            return true;
        if (itemResponse[`${schema.key}_${operator}`] === undefined &&
            !nestjs_microservices_schemas_1.SKIP_ENUMS.includes(operator))
            return true;
        return false;
    }
}
exports.FieldSchemaBuilder = FieldSchemaBuilder;
