"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SchemaValidator = void 0;
const common_1 = require("@nestjs/common");
const comparator_helper_1 = require("./comparator.helper");
const nestjs_microservices_schemas_1 = require("@devseeder/nestjs-microservices-schemas");
const string_helper_1 = require("../string.helper");
const search_encapsulator_helper_1 = require("../search/search-encapsulator.helper");
const get_dynamic_value_service_1 = require("../../service/dynamic/get-dynamic-value.service");
class SchemaValidator {
    constructor(errorService, translationService, entitySchema, entitySchemaData) {
        this.errorService = errorService;
        this.translationService = translationService;
        this.entitySchema = entitySchema;
        this.entitySchemaData = entitySchemaData;
        this.logger = new common_1.Logger(SchemaValidator.name);
    }
    validateRequestSchema(schema, requestMethod, obj, fieldSchemasDb = [], encapsulate = false) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.validateSchema(schema.entity[requestMethod], obj, fieldSchemasDb.filter((schema) => (this.entitySchema.extendedEntities &&
                this.entitySchema.extendedEntities.includes(schema.entity)) ||
                schema.entity === nestjs_microservices_schemas_1.GLOBAL_ENTITY ||
                schema.entity === this.entitySchema.entity), encapsulate);
            if (!['search', 'count', 'groupBy'].includes(requestMethod))
                return;
            yield this.validateFamilySchema(schema.parents, requestMethod, obj, fieldSchemasDb, encapsulate);
            yield this.validateFamilySchema(schema.children, requestMethod, obj, fieldSchemasDb, encapsulate);
        });
    }
    validateFamilySchema(schemaRelation, requestMethod, obj, fieldSchemasDb, encapsulate = false) {
        var _a, e_1, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                for (var _d = true, _e = __asyncValues(Object.keys(schemaRelation)), _f; _f = yield _e.next(), _a = _f.done, !_a; _d = true) {
                    _c = _f.value;
                    _d = false;
                    const relation = _c;
                    const entitySchema = this.entitySchemaData.filter((entSchema) => entSchema.entity === relation);
                    yield this.validateSchema(schemaRelation[relation][requestMethod], search_encapsulator_helper_1.SearchEncapsulatorHelper.buildFamilyEncapsulator(obj, relation), fieldSchemasDb.filter((schema) => (entitySchema[0].extendedEntities &&
                        entitySchema[0].extendedEntities.includes(schema.entity)) ||
                        schema.entity === nestjs_microservices_schemas_1.GLOBAL_ENTITY ||
                        schema.entity === relation), encapsulate);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (!_d && !_a && (_b = _e.return)) yield _b.call(_e);
                }
                finally { if (e_1) throw e_1.error; }
            }
        });
    }
    validateSchema(schema, obj, fieldSchemasDb = [], encapsulate = false) {
        var _a, e_2, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            if (!encapsulate)
                yield this.validateSingleSchema(schema, obj, fieldSchemasDb);
            const arrToValidate = search_encapsulator_helper_1.SearchEncapsulatorHelper.cleanToValidateSchema(obj);
            this.logger.log(`Array To Validate: ${JSON.stringify(arrToValidate)}`);
            try {
                for (var _d = true, arrToValidate_1 = __asyncValues(arrToValidate), arrToValidate_1_1; arrToValidate_1_1 = yield arrToValidate_1.next(), _a = arrToValidate_1_1.done, !_a; _d = true) {
                    _c = arrToValidate_1_1.value;
                    _d = false;
                    const objItem = _c;
                    yield this.validateSingleSchema(schema, objItem, fieldSchemasDb);
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (!_d && !_a && (_b = arrToValidate_1.return)) yield _b.call(arrToValidate_1);
                }
                finally { if (e_2) throw e_2.error; }
            }
        });
    }
    validateSingleSchema(schema, obj, fieldSchemasDb = []) {
        var _a, e_3, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const { error } = schema.validate(obj, {
                messages: yield this.errorService.getJoiErrors(),
            });
            if (error)
                yield this.printSchemaError(error, fieldSchemasDb);
            const validationSchemas = fieldSchemasDb.filter((field) => obj[field.key] !== undefined &&
                field.validations &&
                field.validations.length);
            try {
                for (var _d = true, validationSchemas_1 = __asyncValues(validationSchemas), validationSchemas_1_1; validationSchemas_1_1 = yield validationSchemas_1.next(), _a = validationSchemas_1_1.done, !_a; _d = true) {
                    _c = validationSchemas_1_1.value;
                    _d = false;
                    const fieldSchema = _c;
                    yield this.executeValidationField(fieldSchema, obj);
                }
            }
            catch (e_3_1) { e_3 = { error: e_3_1 }; }
            finally {
                try {
                    if (!_d && !_a && (_b = validationSchemas_1.return)) yield _b.call(validationSchemas_1);
                }
                finally { if (e_3) throw e_3.error; }
            }
        });
    }
    validateEnum(values) {
        return (value, helpers) => {
            if (!values.includes(value)) {
                return helpers.error('any.invalid');
            }
            return value;
        };
    }
    executeValidationField(schema, item) {
        var _a, e_4, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            if (!schema.validations || !schema.validations.length)
                return;
            try {
                for (var _d = true, _e = __asyncValues(schema.validations), _f; _f = yield _e.next(), _a = _f.done, !_a; _d = true) {
                    _c = _f.value;
                    _d = false;
                    const val = _c;
                    const value = get_dynamic_value_service_1.DynamicValueService.getDynamicValue(val.value, val.value, item);
                    const isValid = comparator_helper_1.ComparatorHelper.compare(item[schema.key], value, val.operator, schema.type);
                    if (!isValid) {
                        const compHelper = new comparator_helper_1.ComparatorHelper(this.errorService, this.translationService, [schema.entity]);
                        yield compHelper.getComparatorError(schema.key, val.value, val.operator);
                    }
                }
            }
            catch (e_4_1) { e_4 = { error: e_4_1 }; }
            finally {
                try {
                    if (!_d && !_a && (_b = _e.return)) yield _b.call(_e);
                }
                finally { if (e_4) throw e_4.error; }
            }
        });
    }
    printSchemaError(error, fieldSchemas) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.error(JSON.stringify(error));
            let message = error.message;
            const translate = string_helper_1.StringHelper.extractKey(error.message, /@translate\(([^)]+)\)/);
            if (translate) {
                const translateKey = translate
                    .replaceAll('"', '')
                    .replace(/\[\d+\]/g, '');
                const fieldTranslation = yield this.translationService.getFieldTranslation([fieldSchemas.filter((sch) => sch.key === translateKey)[0].entity], translateKey);
                message = message.replaceAll(`@translate(${translate})`, fieldTranslation.fieldLabel);
            }
            const errorSchema = this.errorService.getJoiErrorByType(error['details'][0].type);
            throw new nestjs_microservices_schemas_1.ErrorSchemaException(string_helper_1.StringHelper.capitalizeFirstLetter(message.replaceAll('"', '')), errorSchema.httpStatus, errorSchema.code, errorSchema.name);
        });
    }
    static removeUndefined(obj) {
        return Object.fromEntries(Object.entries(obj).filter(([_, value]) => value !== undefined));
    }
}
exports.SchemaValidator = SchemaValidator;
