"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var GenericModule_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenericModule = void 0;
const common_1 = require("@nestjs/common");
const mongoose_1 = require("@nestjs/mongoose");
const nestjs_microservices_schemas_1 = require("@devseeder/nestjs-microservices-schemas");
const core_1 = require("@nestjs/core");
const nestjs_microservices_schemas_2 = require("@devseeder/nestjs-microservices-schemas");
const jwt_1 = require("@nestjs/jwt");
const app_constants_1 = require("../app.constants");
const controller_1 = require("../controller");
const dependency_injector_service_1 = require("../injector/dependency-injector.service");
const model_entity_token_injector_1 = require("../injector/model-entity-token.injector");
const config_1 = require("@nestjs/config");
let GenericModule = GenericModule_1 = class GenericModule {
    static forFeature(options) {
        var _a;
        const repositoryProvider = dependency_injector_service_1.DependencyInjectorService.injectRepository(options.entity, options.modelName, options.customProvider);
        return {
            module: GenericModule_1,
            controllers: controllersFactory(options.entity, options.authGuard, options.interceptor, options.customProvider),
            imports: [
                mongoose_1.MongooseModule.forFeature(model_entity_token_injector_1.EntityModelTokenBuilder.buildMongooseStaticModelForFeature(options.modelTokens)),
                nestjs_microservices_schemas_2.SchemasModule.forRootAsync(options.configuration),
                nestjs_microservices_schemas_1.TranslationsModule.forRoot(options.configuration),
                nestjs_microservices_schemas_1.ErrorSchemasModule.forRoot(options.configuration),
                ...(((options === null || options === void 0 ? void 0 : options.customProvider) && ((_a = options === null || options === void 0 ? void 0 : options.customProvider) === null || _a === void 0 ? void 0 : _a.imports)) ||
                    []),
            ],
            providers: [
                repositoryProvider,
                jwt_1.JwtService,
                {
                    provide: app_constants_1.DependencyInjectorToken.SCOPE_KEY,
                    useFactory: (config) => __awaiter(this, void 0, void 0, function* () { return config.get('auth.scopeKey'); }),
                    inject: [config_1.ConfigService],
                },
                options.authGuard,
                GenericModule_1.loadServiceProvider(options.entity, 'get', options.modelTokens, options.customProvider),
                GenericModule_1.loadServiceProvider(options.entity, 'update', options.modelTokens, options.customProvider),
                GenericModule_1.loadServiceProvider(options.entity, 'create', options.modelTokens, options.customProvider),
            ],
            exports: [
                repositoryProvider,
                `GENERIC_GET_SERVICE_${options.entity}`,
                `GENERIC_UPDATE_SERVICE_${options.entity}`,
                `GENERIC_CREATE_SERVICE_${options.entity}`,
            ],
        };
    }
    static loadServiceProvider(entity, providerKey, modelTokens, customProvider) {
        return {
            provide: `GENERIC_${providerKey.toUpperCase()}_SERVICE_${entity}`,
            useFactory: (moduleRef, repository, fieldSchemaData, entitySchemaData, translationService, errorService, request, scopeKey) => __awaiter(this, void 0, void 0, function* () {
                const injectorService = new dependency_injector_service_1.DependencyInjectorService(moduleRef, entitySchemaData, fieldSchemaData, translationService, errorService, request, scopeKey, modelTokens);
                const injectFunction = `inject${providerKey.capitalizeFirstLetter()}Service`;
                const serviceProvider = yield injectorService[injectFunction](entity, repository, customProvider);
                return serviceProvider;
            }),
            inject: [
                core_1.ModuleRef,
                `GENERIC_REPOSITORY_${entity}`,
                nestjs_microservices_schemas_1.SchemaDependecyTokens.FIELD_SCHEMA_DB,
                nestjs_microservices_schemas_1.SchemaDependecyTokens.ENTITY_SCHEMA_DB,
                nestjs_microservices_schemas_1.GetTranslationService,
                nestjs_microservices_schemas_1.ErrorService,
                core_1.REQUEST,
                app_constants_1.DependencyInjectorToken.SCOPE_KEY,
            ],
        };
    }
};
exports.GenericModule = GenericModule;
exports.GenericModule = GenericModule = GenericModule_1 = __decorate([
    (0, common_1.Module)({})
], GenericModule);
function controllersFactory(entity, authGuard, interceptor, customProvider) {
    const controllerArgs = { entity, authGuard, interceptor };
    const genericArr = [
        (0, controller_1.GenericGetController)(controllerArgs),
        (0, controller_1.GenericUpdateController)(controllerArgs),
        (0, controller_1.GenericCreateController)(controllerArgs),
    ];
    if (checkCustomController('get', customProvider))
        genericArr.push(customProvider.controller.get);
    if (checkCustomController('update', customProvider))
        genericArr.push(customProvider.controller.update);
    if (checkCustomController('create', customProvider))
        genericArr.push(customProvider.controller.get);
    return genericArr;
}
function checkCustomController(ctrlKey, customProvider) {
    return (customProvider &&
        customProvider.controller &&
        customProvider.controller[ctrlKey]);
}
