/// <reference types="mongoose/types/aggregate" />
/// <reference types="mongoose/types/callback" />
/// <reference types="mongoose/types/collection" />
/// <reference types="mongoose/types/connection" />
/// <reference types="mongoose/types/cursor" />
/// <reference types="mongoose/types/document" />
/// <reference types="mongoose/types/error" />
/// <reference types="mongoose/types/expressions" />
/// <reference types="mongoose/types/helpers" />
/// <reference types="mongoose/types/middlewares" />
/// <reference types="mongoose/types/indexes" />
/// <reference types="mongoose/types/models" />
/// <reference types="mongoose/types/mongooseoptions" />
/// <reference types="mongoose/types/pipelinestage" />
/// <reference types="mongoose/types/populate" />
/// <reference types="mongoose/types/query" />
/// <reference types="mongoose/types/schemaoptions" />
/// <reference types="mongoose/types/schematypes" />
/// <reference types="mongoose/types/session" />
/// <reference types="mongoose/types/types" />
/// <reference types="mongoose/types/utility" />
/// <reference types="mongoose/types/validation" />
/// <reference types="mongoose/types/virtuals" />
/// <reference types="mongoose/types/inferschematype" />
import { Logger } from '@nestjs/common';
import { ClientSession, HydratedDocument, Model, ObjectId } from 'mongoose';
export type MongooseDocument = HydratedDocument<any>;
export type MongooseDocumentID = string | ObjectId;
export declare abstract class MongooseRepository<Collection, MongooseModel> {
    protected model: Model<MongooseModel>;
    protected readonly logger: Logger;
    protected session: ClientSession;
    constructor(model: Model<MongooseModel>);
    insertOne(item: Collection, name: string, session?: ClientSession): Promise<ObjectId>;
    create(document: Collection, session?: ClientSession): Promise<MongooseDocument>;
    groupBy(group: any, match?: {}, select?: {}): Promise<any[]>;
    aggregate(group: any, match?: {}, select?: {}, unwind?: string): Promise<any[]>;
    aggregateNotExists(from: string, joinFrom: string, joinLet: string, match?: any): Promise<any[]>;
    findAll(select?: object): Promise<any[]>;
    findById(id: MongooseDocumentID, select?: object): Promise<MongooseDocument>;
    updateOneById(id: MongooseDocumentID, data: any, session?: ClientSession, pushData?: {}, strict?: boolean): Promise<void>;
    updateOne(query: any, data: any, pushData?: {}, strict?: boolean, session?: ClientSession): Promise<void>;
    updateMany(query: any, data: any, pushData?: {}, strict?: boolean, session?: ClientSession): Promise<any>;
    deleteOneById(id: string | number): Promise<void>;
    find(searchParams: any, select?: any, sort?: any, regex?: boolean, limit?: number, skip?: number): Promise<any[]>;
    getIndexes(): object;
    count(searchParams: object): Promise<number>;
    private startSession;
    private endSession;
    startTransaction(): Promise<ClientSession>;
    commit(): Promise<void>;
    rollback(): Promise<void>;
    private getSessionOptions;
}
