"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenericCreateService = void 0;
const common_1 = require("@nestjs/common");
const abstract_db_service_1 = require("./abstract-db.service");
const microservices_exceptions_1 = require("@devseeder/microservices-exceptions");
const nestjs_microservices_schemas_1 = require("@devseeder/nestjs-microservices-schemas");
const core_1 = require("@nestjs/core");
const error_keys_enum_1 = require("../../enum/error-keys.enum");
const string_helper_1 = require("../../helper/string.helper");
const generic_repository_1 = require("../../mongoose/generic.repository");
const app_constants_1 = require("../../app.constants");
let GenericCreateService = class GenericCreateService extends abstract_db_service_1.AbstractDBService {
    constructor(repository, entity, fieldSchemaData, entitySchemaData, translationService, errorService, request, scopeKey) {
        super(repository, entity, fieldSchemaData, entitySchemaData, translationService, errorService, request);
        this.repository = repository;
        this.entity = entity;
        this.fieldSchemaData = fieldSchemaData;
        this.entitySchemaData = entitySchemaData;
        this.translationService = translationService;
        this.errorService = errorService;
        this.request = request;
        this.scopeKey = scopeKey;
    }
    create(body, validateInput = true, session = null) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.convertRelation(body, validateInput);
            const bodyCreate = Object.assign(Object.assign({}, this.getDynamicValues(body)), { active: true });
            this.logger.log(`Body: ${JSON.stringify(bodyCreate)}`);
            try {
                const insertedId = yield this.repository.insertOne(bodyCreate, this.entitySchema.itemLabel, session);
                return { _id: insertedId };
            }
            catch (err) {
                if (err instanceof microservices_exceptions_1.MongoDBException) {
                    if (err.errCode === 11000) {
                        const entityTranslation = yield this.translationService.getEntityTranslation(this.entitySchema.entity);
                        const indexKey = err.getResponse()
                            .replace('E11000 duplicate key error collection: ', '')
                            .split(' index: ')[1]
                            .split('_')[0];
                        this.errorService.throwError(error_keys_enum_1.ErrorKeys.ALREADY_EXISTS, {
                            key: entityTranslation.itemLabel,
                            value: body[indexKey],
                        });
                    }
                }
            }
        });
    }
    cloneOne(id, changeUniqueIndex = true, relationsToClone = [], cloneBody = {}, session = null, validateInput = true) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.log(`Cloning ${this.entitySchema.itemLabel} '${id}'...`);
            const cloneTarget = yield this.validateId(id);
            yield this.convertRelation(cloneBody, validateInput);
            const bodyCreate = Object.assign(Object.assign({}, cloneTarget), { active: true });
            if (changeUniqueIndex)
                yield this.getUniqueIndexToClone(bodyCreate);
            this.logger.log(`CloneBody ${JSON.stringify(cloneBody)}`);
            this.logger.log(`Body ${JSON.stringify(Object.assign(Object.assign({}, bodyCreate), cloneBody))}`);
            const insertedId = yield this.create(Object.assign(Object.assign({}, bodyCreate), cloneBody), false, session);
            yield this.cloneRelations(id, insertedId._id.toString(), relationsToClone, session);
            return insertedId;
        });
    }
    cloneByIds(ids, relationsToClone = [], cloneBody = {}) {
        var _a, ids_1, ids_1_1;
        var _b, e_1, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.log(`Cloning ${this.entitySchema.itemLabel} '${ids.join(',')}'...`);
            const arrClone = [];
            try {
                const session = yield this.repository.startTransaction();
                try {
                    for (_a = true, ids_1 = __asyncValues(ids); ids_1_1 = yield ids_1.next(), _b = ids_1_1.done, !_b; _a = true) {
                        _d = ids_1_1.value;
                        _a = false;
                        const id = _d;
                        const insertedId = yield this.cloneOne(id, true, relationsToClone, cloneBody, session);
                        arrClone.push(insertedId._id);
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (!_a && !_b && (_c = ids_1.return)) yield _c.call(ids_1);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
                yield this.repository.commit();
                this.logger.log('Clonning proccess finished');
            }
            catch (err) {
                this.logger.error(`Error while clonning, starting rollback for all itens`);
                yield this.repository.rollback();
                this.logger.error('Rollback finished for all itens');
                throw err;
            }
            return { _ids: arrClone };
        });
    }
    cloneRelations(cloneId, newId, relationsToClone = [], session = null) {
        var _a, e_2, _b, _c, _d, e_3, _e, _f;
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.log('Cloning relations ');
            const relations = this.entitySchema.subRelations.filter((sub) => relationsToClone.length
                ? (relationsToClone.includes(sub.entity) ||
                    relationsToClone[0] === '*') &&
                    sub.clone
                : false);
            this.logger.log(`Cloning ${relations.length} relations`);
            try {
                for (var _g = true, relations_1 = __asyncValues(relations), relations_1_1; relations_1_1 = yield relations_1.next(), _a = relations_1_1.done, !_a; _g = true) {
                    _c = relations_1_1.value;
                    _g = false;
                    const rel = _c;
                    this.logger.log(`Cloning relation '${rel.service}'...`);
                    this.logger.log(`Service relation 'get${rel.entity.capitalizeFirstLetter()}Service'...`);
                    const data = yield this[`get${rel.entity.capitalizeFirstLetter()}Service`].search({
                        [rel.key]: cloneId,
                        select: 'name',
                    }, false, false);
                    const { items: subItems } = data;
                    this.logger.log(`${subItems.length} itens found for '${rel.entity}'...`);
                    try {
                        for (var _h = true, subItems_1 = (e_3 = void 0, __asyncValues(subItems)), subItems_1_1; subItems_1_1 = yield subItems_1.next(), _d = subItems_1_1.done, !_d; _h = true) {
                            _f = subItems_1_1.value;
                            _h = false;
                            const item = _f;
                            yield this[`create${rel.entity.capitalizeFirstLetter()}Service`].cloneOne(item._id, false, '*', {
                                [rel.key]: rel.array ? [newId] : newId,
                            }, session, false);
                        }
                    }
                    catch (e_3_1) { e_3 = { error: e_3_1 }; }
                    finally {
                        try {
                            if (!_h && !_d && (_e = subItems_1.return)) yield _e.call(subItems_1);
                        }
                        finally { if (e_3) throw e_3.error; }
                    }
                    this.logger.log(`Relation '${rel.service}' successfully cloned!`);
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (!_g && !_a && (_b = relations_1.return)) yield _b.call(relations_1);
                }
                finally { if (e_2) throw e_2.error; }
            }
        });
    }
    getUniqueIndexToClone(bodyCreate) {
        return __awaiter(this, void 0, void 0, function* () {
            const objIndexes = yield this.repository.getIndexes();
            delete objIndexes['_id_'];
            const mapKeys = Object.values(objIndexes).map((key) => key[0][0]);
            const randomId = string_helper_1.StringHelper.generateRandomString(7);
            [...mapKeys, ...(this.entitySchema.copyFields || [])].forEach((key) => {
                if (typeof bodyCreate[key] == 'string')
                    bodyCreate[key] = `${bodyCreate[key]}(copy ${randomId})`;
            });
            delete bodyCreate['_id'];
            delete bodyCreate['__v'];
            delete bodyCreate['createdAt'];
            delete bodyCreate['updatedAt'];
            delete bodyCreate['inactivationDate'];
            delete bodyCreate['inactivationReason'];
        });
    }
};
exports.GenericCreateService = GenericCreateService;
exports.GenericCreateService = GenericCreateService = __decorate([
    (0, common_1.Injectable)({ scope: common_1.Scope.REQUEST }),
    __param(6, (0, common_1.Inject)(core_1.REQUEST)),
    __param(7, (0, common_1.Inject)(app_constants_1.DependencyInjectorToken.SCOPE_KEY)),
    __metadata("design:paramtypes", [generic_repository_1.GenericRepository, String, Array, Array, nestjs_microservices_schemas_1.GetTranslationService,
        nestjs_microservices_schemas_1.ErrorService,
        Request, String])
], GenericCreateService);
