"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenericGetService = void 0;
const common_1 = require("@nestjs/common");
const sort_helper_1 = require("../../helper/search/sort.helper");
const microservices_exceptions_1 = require("@devseeder/microservices-exceptions");
const abstract_search_service_1 = require("./abstract-search.service");
const nestjs_microservices_schemas_1 = require("@devseeder/nestjs-microservices-schemas");
const core_1 = require("@nestjs/core");
const error_keys_enum_1 = require("../../enum/error-keys.enum");
const generic_repository_1 = require("../../mongoose/generic.repository");
const app_constants_1 = require("../../app.constants");
let GenericGetService = class GenericGetService extends abstract_search_service_1.AbstractSearchService {
    constructor(repository, entity, fieldSchemaData = [], entitySchemaData = [], translationService, errorService, request, scopeKey) {
        super(repository, entity, fieldSchemaData, entitySchemaData, translationService, errorService, request);
        this.repository = repository;
        this.entity = entity;
        this.fieldSchemaData = fieldSchemaData;
        this.entitySchemaData = entitySchemaData;
        this.translationService = translationService;
        this.errorService = errorService;
        this.request = request;
        this.scopeKey = scopeKey;
    }
    search(searchParams = null, paginate = true, convertOutput = true) {
        return __awaiter(this, void 0, void 0, function* () {
            let searchWhere;
            try {
                this.logger.log(`Build Search Params: ${JSON.stringify(searchParams)}`);
                searchWhere = yield this.buildSearchParams(searchParams);
            }
            catch (err) {
                if (err instanceof microservices_exceptions_1.CustomErrorException)
                    if (err.errCode === 404)
                        return { items: [] };
                throw err;
            }
            const { page, pageSize } = this.getPagination(searchParams, searchWhere);
            const { sort, sortExternal, hasExternal } = this.validateOrderField(searchWhere, searchParams === null || searchParams === void 0 ? void 0 : searchParams.orderBy, (searchParams === null || searchParams === void 0 ? void 0 : searchParams.orderMode) | 1);
            this.logger.log(`Searching '${this.entitySchema.itemLabel}' ${JSON.stringify(searchWhere)}...`);
            const responseItems = yield this.repository.find(searchWhere, this.getSelectFields(searchParams), sort, false, pageSize, page);
            this.logger.log(`Items found: ${responseItems.length}.`);
            yield this.validateOnlyLoggedUserForItems(responseItems);
            const arrMap = yield responseItems.map((item) => __awaiter(this, void 0, void 0, function* () { return yield this.convertRelation(item); }));
            const totalSorted = sort_helper_1.SortHelper.orderBy(convertOutput ? yield Promise.all(arrMap) : responseItems, this.fieldSchemaDb, sortExternal, hasExternal);
            const response = {
                items: totalSorted,
            };
            if (paginate)
                response['meta'] = yield this.getPaginatedResponse(totalSorted, searchWhere, searchParams.page, pageSize);
            return response;
        });
    }
    getById(id, validateError = false) {
        return __awaiter(this, void 0, void 0, function* () {
            let item;
            try {
                this.logger.log(`Get ${this.entitySchema.entity} By Id ${id}`);
                item = yield this.repository.findById(id, { all: 0 });
                if (!item && validateError) {
                    const entityTranslation = yield this.translationService.getEntityTranslation(this.entitySchema.entity);
                    this.errorService.throwError(error_keys_enum_1.ErrorKeys.NOT_FOUND, {
                        key: entityTranslation.itemLabel,
                    });
                }
            }
            catch (err) {
                this.errorService.throwError(error_keys_enum_1.ErrorKeys.INVALID_DATA, {
                    key: 'Id',
                    value: id,
                });
            }
            yield this.validateOnlyLoggedUserForItems([item]);
            return this.convertRelation(item);
        });
    }
    getForm() {
        var _a, e_1, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const fields = this.fieldSchemaDb;
            const orderFields = this.fieldSchemaDb.filter((fields) => fields.orderBy);
            const arrayResponse = [];
            try {
                for (var _d = true, fields_1 = __asyncValues(fields), fields_1_1; fields_1_1 = yield fields_1.next(), _a = fields_1_1.done, !_a; _d = true) {
                    _c = fields_1_1.value;
                    _d = false;
                    const field = _c;
                    const objectItem = Object.assign({}, field);
                    if (field.type === 'externalId' && !field.hidden) {
                        const values = yield this[`get${field.externalRelation.service.capitalizeFirstLetter()}Service`].search({
                            pageSize: 50,
                            select: field.externalRelation['nameKey']
                                ? field.externalRelation['nameKey']
                                : 'name,icon',
                        });
                        objectItem['values'] = values.items;
                    }
                    objectItem['translation'] =
                        yield this.translationService.getFieldTranslation(this.entityLabels, field.key);
                    arrayResponse.push(objectItem);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (!_d && !_a && (_b = fields_1.return)) yield _b.call(fields_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
            const cloneRelations = this.entitySchema.subRelations.filter((sub) => sub.clone);
            const entityTranslation = yield this.translationService.getEntityTranslation(this.entitySchema.entity);
            const response = {
                fields: arrayResponse,
                cloneRelations: yield Promise.all(cloneRelations.map((rel) => __awaiter(this, void 0, void 0, function* () {
                    return ({
                        relation: rel.entity,
                        label: (yield this.translationService.getFieldTranslation([rel.entity], rel.key)).fieldLabel,
                    });
                }))),
                entityRefs: Object.assign({ translations: entityTranslation }, this.entitySchema),
            };
            response.filterOptions = {
                orderBy: yield Promise.all(orderFields.map((field) => __awaiter(this, void 0, void 0, function* () {
                    return ({
                        key: field.key,
                        label: (yield this.translationService.getFieldTranslation(this.entityLabels, field.key)).fieldLabel,
                    });
                }))),
            };
            return response;
        });
    }
    getPagination(search = null, searchWhere) {
        if (!search)
            return { page: 0, pageSize: 0 };
        delete searchWhere.page;
        delete searchWhere.pageSize;
        const limit = (search === null || search === void 0 ? void 0 : search.pageSize) | 0;
        const curPage = (search === null || search === void 0 ? void 0 : search.page) <= 1 ? 0 : search === null || search === void 0 ? void 0 : search.page;
        const returnPagination = {
            pageSize: (search === null || search === void 0 ? void 0 : search.pageSize) | 0,
            page: limit && curPage ? (curPage - 1) * limit + 1 : 0,
        };
        this.logger.log(`Pagination ${JSON.stringify(returnPagination)}...`);
        return returnPagination;
    }
    getSelectFields(params) {
        if (!params.select)
            return { all: 0 };
        const selectParam = {};
        params.select.split(',').forEach((key) => {
            if (!this.fieldSchemaDb.find((schema) => schema.key === key) &&
                !['icon'].includes(key))
                this.errorService.throwError(error_keys_enum_1.ErrorKeys.INVALID_DATA, {
                    key: 'select field',
                    value: key,
                });
            selectParam[key] = 1;
        });
        return Object.assign(Object.assign({}, selectParam), { _id: 1 });
    }
    getPaginatedResponse(items, searchWhere, page, pageSize) {
        return __awaiter(this, void 0, void 0, function* () {
            const count = yield this.repository.count(searchWhere);
            const hasNextPage = (page - 1) * pageSize + items.length < count;
            const currentPage = page ? Number(page) : 1;
            return {
                currentPage,
                nextPage: hasNextPage ? currentPage + 1 : currentPage,
                hasNext: hasNextPage,
                pageRecords: items.length,
                totalRecords: count,
                actualIndex: items.length ? (page - 1) * pageSize + items.length : 0,
                numberOfPages: Math.ceil(count / pageSize),
            };
        });
    }
    count(searchParams) {
        return __awaiter(this, void 0, void 0, function* () {
            const searchWhere = yield this.buildSearchParams(searchParams);
            this.logger.log(`Counting '${this.entitySchema.itemLabel}' for: ${JSON.stringify(searchWhere)}`);
            const totalRecords = yield this.repository.count(searchWhere);
            yield this.validateLoggedUserByCount(totalRecords, searchWhere);
            return {
                totalRecords,
            };
        });
    }
    groupBy(groupedEntity, searchParams = null) {
        var _a, e_2, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const sumField = searchParams['sumField'];
            delete searchParams['sumField'];
            const rel = this.fieldSchemaDb.filter((schema) => schema.externalRelation &&
                schema.externalRelation.service === groupedEntity);
            const sumFieldSchema = this.fieldSchemaDb.filter((schema) => schema.key === sumField &&
                ['number', 'double', 'integer'].includes(schema.type));
            if (sumField && !sumFieldSchema.length)
                this.errorService.throwError(error_keys_enum_1.ErrorKeys.INVALID_DATA, {
                    key: 'Sum Field',
                    value: sumField,
                });
            if (!rel.length)
                this.errorService.throwError(error_keys_enum_1.ErrorKeys.INVALID_DATA, {
                    key: 'Relation',
                    value: groupedEntity,
                });
            yield this.count(searchParams);
            const searchWhere = yield this.buildSearchParams(searchParams);
            const aggResponse = yield this.repository.groupBy(searchWhere, rel[0].externalRelation.service, rel[0].key, sumField, rel[0].array);
            const arrResponse = [];
            try {
                for (var _d = true, aggResponse_1 = __asyncValues(aggResponse), aggResponse_1_1; aggResponse_1_1 = yield aggResponse_1.next(), _a = aggResponse_1_1.done, !_a; _d = true) {
                    _c = aggResponse_1_1.value;
                    _d = false;
                    const agg = _c;
                    if (!agg._id || !agg._id.length)
                        continue;
                    const responseObj = {};
                    responseObj[`${rel[0].key}`] = {
                        id: agg._id[0],
                        value: agg.name[0],
                    };
                    arrResponse.push(Object.assign(Object.assign({}, responseObj), { groupResult: {
                            totalSum: sumField
                                ? agg[`total${sumField.capitalizeFirstLetter()}`]
                                : undefined,
                            avg: sumField
                                ? agg[`avg${sumField.capitalizeFirstLetter()}`]
                                : undefined,
                            count: agg.count,
                        } }));
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (!_d && !_a && (_b = aggResponse_1.return)) yield _b.call(aggResponse_1);
                }
                finally { if (e_2) throw e_2.error; }
            }
            return arrResponse;
        });
    }
};
exports.GenericGetService = GenericGetService;
exports.GenericGetService = GenericGetService = __decorate([
    (0, common_1.Injectable)({ scope: common_1.Scope.REQUEST }),
    __param(6, (0, common_1.Inject)(core_1.REQUEST)),
    __param(7, (0, common_1.Inject)(app_constants_1.DependencyInjectorToken.SCOPE_KEY)),
    __metadata("design:paramtypes", [generic_repository_1.GenericRepository, String, Array, Array, nestjs_microservices_schemas_1.GetTranslationService,
        nestjs_microservices_schemas_1.ErrorService,
        Request, String])
], GenericGetService);
