"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenericUpdateService = void 0;
const common_1 = require("@nestjs/common");
const abstract_search_service_1 = require("./abstract-search.service");
const nestjs_microservices_schemas_1 = require("@devseeder/nestjs-microservices-schemas");
const core_1 = require("@nestjs/core");
const error_keys_enum_1 = require("../../enum/error-keys.enum");
const helper_1 = require("../../helper");
const inactivation_reason_enum_1 = require("../../enum/inactivation-reason.enum");
const generic_repository_1 = require("../../mongoose/generic.repository");
const app_constants_1 = require("../../app.constants");
let GenericUpdateService = class GenericUpdateService extends abstract_search_service_1.AbstractSearchService {
    constructor(repository, entity, fieldSchemaData, entitySchemaData, translationService, errorService, request, reflector, scopeKey) {
        super(repository, entity, fieldSchemaData, entitySchemaData, translationService, errorService, request);
        this.repository = repository;
        this.entity = entity;
        this.fieldSchemaData = fieldSchemaData;
        this.entitySchemaData = entitySchemaData;
        this.translationService = translationService;
        this.errorService = errorService;
        this.request = request;
        this.reflector = reflector;
        this.scopeKey = scopeKey;
    }
    updateById(id, body, session = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!body || !Object.keys(body).length)
                this.errorService.throwError(error_keys_enum_1.ErrorKeys.EMPTY_BODY);
            this.logger.log(`Updating record by id '${id}'`);
            yield this.validateId(id);
            yield this.convertRelation(body);
            this.logger.log(`Body: ${JSON.stringify(body)}`);
            yield this.repository.updateOneById(id, body, session);
        });
    }
    activation(id, activation, cascadeRelations = '', reason = inactivation_reason_enum_1.InactivationReason.MANUAL) {
        return __awaiter(this, void 0, void 0, function* () {
            const body = {
                active: activation,
                inactivationReason: !activation ? reason : null,
                inactivationDate: !activation ? helper_1.DateHelper.getDateNow() : null,
            };
            const session = yield this.repository.startTransaction();
            try {
                yield this.updateById(id, body, session);
                if (!cascadeRelations.length) {
                    this.logger.log(`No cascade relations`);
                    yield this.repository.commit();
                    return;
                }
                yield this.cascadeUpdate(id, body, !activation, cascadeRelations, session);
                yield this.repository.commit();
            }
            catch (err) {
                yield this.repository.rollback();
                throw err;
            }
        });
    }
    cascadeUpdate(id, body, active, cascadeRelations = '*', session = null) {
        var _a, e_1, _b, _c, _d, e_2, _e, _f;
        return __awaiter(this, void 0, void 0, function* () {
            const relations = this.entitySchema.subRelations.filter((ent) => cascadeRelations === '*' ||
                (cascadeRelations && cascadeRelations.split(',').includes(ent.entity)));
            if (cascadeRelations && cascadeRelations.length && cascadeRelations !== '*')
                cascadeRelations.split(',').forEach((rel) => {
                    const foundRelation = this.entitySchema.subRelations.filter((sub) => sub.entity === rel);
                    if (!foundRelation.length)
                        this.errorService.throwError(error_keys_enum_1.ErrorKeys.INVALID_DATA, {
                            key: 'Relation',
                            value: rel,
                        });
                });
            try {
                for (var _g = true, relations_1 = __asyncValues(relations), relations_1_1; relations_1_1 = yield relations_1.next(), _a = relations_1_1.done, !_a; _g = true) {
                    _c = relations_1_1.value;
                    _g = false;
                    const rel = _c;
                    this.logger.log(`Service relation 'get${rel.entity.capitalizeFirstLetter()}Service'...`);
                    const data = yield this[`get${rel.entity.capitalizeFirstLetter()}Service`].search({
                        [rel.key]: id,
                        select: 'name',
                        active,
                    }, false, false);
                    const { items: subItems } = data;
                    this.logger.log(`${subItems.length} itens found for '${rel.entity}'...`);
                    try {
                        for (var _h = true, subItems_1 = (e_2 = void 0, __asyncValues(subItems)), subItems_1_1; subItems_1_1 = yield subItems_1.next(), _d = subItems_1_1.done, !_d; _h = true) {
                            _f = subItems_1_1.value;
                            _h = false;
                            const item = _f;
                            this.logger.log(`Updating ${item._id} ${JSON.stringify(body)}...`);
                            yield this[`update${rel.entity.capitalizeFirstLetter()}Service`].updateById(item._id, body, session);
                        }
                    }
                    catch (e_2_1) { e_2 = { error: e_2_1 }; }
                    finally {
                        try {
                            if (!_h && !_d && (_e = subItems_1.return)) yield _e.call(subItems_1);
                        }
                        finally { if (e_2) throw e_2.error; }
                    }
                    this.logger.log(`Relation '${rel.service}' successfully updated!`);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (!_g && !_a && (_b = relations_1.return)) yield _b.call(relations_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
        });
    }
    updateBy(search, body) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!body || !Object.keys(body).length)
                this.errorService.throwError(error_keys_enum_1.ErrorKeys.EMPTY_BODY);
            this.logger.log(`Updating records by '${JSON.stringify(search)}'`);
            const searchWhere = yield this.buildSearchParams(search);
            const items = yield this.repository.count(searchWhere);
            if (items <= 0)
                this.errorService.throwError(error_keys_enum_1.ErrorKeys.NO_RECORD_UPDATE);
            this.logger.log(`Body: ${JSON.stringify(body)}`);
            yield this.validateLoggedUserByCount(items, searchWhere);
            yield this.convertRelation(body);
            try {
                yield this.repository.updateMany(searchWhere, body);
            }
            catch (err) {
                this.errorService.throwError(error_keys_enum_1.ErrorKeys.ALREADY_UPDATED);
            }
        });
    }
};
exports.GenericUpdateService = GenericUpdateService;
exports.GenericUpdateService = GenericUpdateService = __decorate([
    (0, common_1.Injectable)({ scope: common_1.Scope.REQUEST }),
    __param(6, (0, common_1.Inject)(core_1.REQUEST)),
    __param(8, (0, common_1.Inject)(app_constants_1.DependencyInjectorToken.SCOPE_KEY)),
    __metadata("design:paramtypes", [generic_repository_1.GenericRepository, String, Array, Array, nestjs_microservices_schemas_1.GetTranslationService,
        nestjs_microservices_schemas_1.ErrorService,
        Request,
        core_1.Reflector, String])
], GenericUpdateService);
