"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DynamicValueService = void 0;
const dynamic_values_enum_1 = require("../../enum/dynamic-values.enum");
const helper_1 = require("../../helper");
class DynamicValueService {
    constructor(translationService) {
        this.translationService = translationService;
    }
    static getDynamicValue(dynamicValue, value, item = {}) {
        if (typeof dynamicValue !== 'string')
            return value;
        if (value.startsWith('$'))
            return item[value.replace('$', '')];
        switch (dynamicValue) {
            case dynamic_values_enum_1.DynamicValuesEnum.CUR_DATETIME:
                return helper_1.DateHelper.getDateNow();
            case dynamic_values_enum_1.DynamicValuesEnum.CUR_DATE:
                return helper_1.DateHelper.formatDate(helper_1.DateHelper.getDateNow(), 'YYYY-MM-DD');
            case dynamic_values_enum_1.DynamicValuesEnum.CUR_TIME:
                return helper_1.DateHelper.formatDate(helper_1.DateHelper.getDateNow(), 'HH:mm:ss');
            case dynamic_values_enum_1.DynamicValuesEnum.CUR_USER:
                return 1;
            default:
                return DynamicValueService.testRegexDynamicValues(value);
        }
    }
    static testRegexDynamicValues(value) {
        if (DynamicValueService.testRegex(dynamic_values_enum_1.DynamicValuesEnum.REGEX_CALCULATE, value))
            return 0;
        return value;
    }
    static testRegex(dynamicValue, value) {
        const regex = new RegExp(dynamicValue);
        return regex.test(value);
    }
    getValueMessage(dynamicValue, value) {
        if (typeof dynamicValue !== 'string')
            return value;
        if (value.startsWith('$'))
            return value.replace('$', '');
        switch (dynamicValue) {
            case dynamic_values_enum_1.DynamicValuesEnum.CUR_DATETIME:
            case dynamic_values_enum_1.DynamicValuesEnum.CUR_DATE:
            case dynamic_values_enum_1.DynamicValuesEnum.CUR_TIME:
            case dynamic_values_enum_1.DynamicValuesEnum.CUR_USER:
                return this.translationService.getServiceKeyTranslation(dynamicValue);
            default:
                return value;
        }
    }
}
exports.DynamicValueService = DynamicValueService;
